/**
 * \file hardwareSensorAbstract.cpp
 * \date 15/03/2011
 * \author croussil
 * \ingroup rtslam
 */


#include "rtslam/hardwareSensorAbstract.hpp"

namespace jafar {
namespace rtslam {
namespace hardware {

  //enum Quantity { qPos, qOriQuat, qOriEuler, qVel, qAbsVel, qNormVel, qAngVel, qAbsAngVel, qAcc, qAbsAcc, qBundleobs, qMag, qNQuantity };
  const int HardwareSensorProprioAbstract::QuantityDataSizes[qNQuantity] = { 3, 4, 3, 3, 3, 1, 3, 3, 3, 3, 6, 3 };
  const int HardwareSensorProprioAbstract::QuantityObsSizes [qNQuantity] = { 3, 3, 3, 3, 3, 1, 3, 3, 3, 3, 3, 3 };


#ifdef HAVE_POSTERLIB

STATUS reentrantPosterIoctl(POSTER_ID posterId, int code, void *parg)
{
  static boost::mutex m;
  m.lock();
  STATUS res = posterIoctl(posterId, code, parg);
  m.unlock();
  return res;
}

int reentrantPosterRead(POSTER_ID posterId, int offset, void *buf, int nbytes)
{
  static boost::mutex m;
  m.lock();
  int res = posterRead(posterId, offset, buf, nbytes);
  m.unlock();
  return res;
}

#endif


bool checkStream(std::istream &f, std::string const & owner)
{
	if (f.fail() && !f.eof())
	{
		std::cerr << owner << ": stream error '" << (f.bad() ? "bad" : "fail") << "', failed to read ";
		f.setstate(f.rdstate() & (~(std::ios_base::failbit|std::ios_base::badbit)));
		std::string line; std::getline(f,line);
		std::cerr << line << " ; skipping." << std::endl;
		return false;
	}
	return true;
}


void LoggableProprio::log()
{
	// binary
	for(unsigned i = 0; i < data.size(); ++i)
		fb.write((char*)&(data(i)), sizeof(double));

	// new text tab separated
	fl << std::setprecision(19) << data(0) << std::setprecision(10);
	for(unsigned i = 1; i < data.size(); ++i) fl << "\t" << data(i);
	fl << std::endl;

	// old text ublas::vector
	//f << std::setprecision(19) << data << std::endl;
}


void ProprioLog::openWrite(std::string name, std::string dump_path, int format, int size, kernel::LoggerTask *loggerTask)
{
	/// binary format : int size; double[size][n] readings;
	/// text format : reading[0]\treading[1]...\nreading[0]...

	std::ostringstream oss; oss << dump_path << "/" << name;
	std::ostringstream ossl; ossl << oss.str() << ".log";
	std::ostringstream ossb; ossb << oss.str() << ".dat";
std::cout << "openWrite " << name << " files " << ossl.str() << " and " << ossb.str() << std::endl;
	fl.open(ossl.str().c_str(), std::ios_base::out);
	fl << "#" << format << "\t" << size << std::endl;
	fb.open(ossb.str().c_str(), std::ios_base::out | std::ios_base::binary);
	fb.write((char*)&format, sizeof(int));
	fb.write((char*)&size, sizeof(int));
	this->size = size;
	this->name = "ProprioLog " + name;
	this->loggerTask = loggerTask;
}


void ProprioLog::openRead(std::string name, std::string dump_path, int &format, int &size)
{
	std::ostringstream oss; oss << dump_path << "/" << name;
	std::ostringstream ossl; ossl << oss.str() << ".log";
	std::ostringstream ossb; ossb << oss.str() << ".dat";
	fb.open(ossb.str().c_str(), std::ios_base::in | std::ios_base::binary);
	if (fb.fail())
	{
		binary = false;
		fl.open(ossl.str().c_str(), std::ios_base::in);
		if (fl.fail()) { std::cerr << "Cannot open file " << ossl.str() << std::endl; return; }
		else
		{
			std::cerr << "Using log " << ossl.str() << std::endl;
			char tmpc; fl >> tmpc;
			if (tmpc == '#') { fl >> format >> size; }
			else { old = true; fl >> size; fl.seekg(0); }
		}
	} else
	{
		fb.read((char*)&format, sizeof(int));
		fb.read((char*)&size, sizeof(int));
		std::cerr << "Using log " << ossb.str() << std::endl;
	}
	this->size = size;
	this->name = "ProprioLog " + name;
}


bool ProprioLog::read(jblas::vec &v)
{
	if (v.size() != (unsigned)size) v.resize(size);
	if (binary)
	{
		for(int i = 0; i < size; ++i) fb.read((char*)&(v(i)), sizeof(double));
		if (fb.eof()) return true;
	} else
	{
		if (old) fl >> v;
		else for(int i = 0; i < size; ++i)	fl >> v(i);
		if (!checkStream(fl,name)) read(v);
	}
	return false;
}


void ProprioLog::close()
{
	if (loggerTask) loggerTask->join();
	if (fl.is_open()) fl.close();
	if (fb.is_open()) fb.close();
}


}}}
