/**
 * \file HardwareSensorCamera.cpp
 * \date 18/06/2010
 * \author croussil
 * \ingroup rtslam
 */

#include <algorithm>
#include <sstream>
#include <fstream>

#if 0
// creates conflict with boost sandbox with boost 1.42 in debug
#include <boost/bind.hpp>
#include <boost/filesystem.hpp>
#include <boost/regex.hpp>
#endif

#include "kernel/timingTools.hpp"
#include "rtslam/hardwareSensorCamera.hpp"


#include <image/Image.hpp>

namespace jafar {
namespace rtslam {
namespace hardware {

  IdFactory HardwareSensorCamera::cameraIds;

	void HardwareSensorCamera::preloadTaskOffline(void)
	{ JFR_GLOBAL_TRY
		int ndigit = 0;
		unsigned findex;

		std::ostringstream oss; oss << dump_path << "/images";
		if (cameraIds.countUsed() > 1) oss << "_" << id();
		oss << ".dat";
		std::fstream fb(oss.str().c_str(), std::ios_base::in | std::ios_base::binary);
		bool binary = !fb.fail();

		while(!stopping)
		{
			// acquire the image
			boost::unique_lock<boost::mutex> l(mutex_data);
			while (isFull(true) && !stopping) cond_offline_freed.wait(l);
			l.unlock();
			if (stopping) break;
			int buff_write = getWritePos();
			while (!stopping)
			{
				// FIXME manage multisensors : put sensor id in filename
				std::ostringstream oss;
				for (int i = 3; i <= 7; ++i)
				{
					if (!found_first) ndigit = i;
					oss.str(""); oss << dump_path << "/image_";
					if (cameraIds.countUsed() > 1) oss << id() << "_";
					oss << std::setw(ndigit) << std::setfill('0') << index_load;
					if (found_first != 2 && bufferSpecPtr[buff_write]->img->load(oss.str() + std::string(".pgm"), 0) && found_first == 0) { found_first = 1; std::cout << "First image " << oss.str() << ".pgm" << std::endl; }
					if (found_first != 1 && bufferSpecPtr[buff_write]->img->load(oss.str() + std::string(".png"), 0) && found_first == 0) { found_first = 2; std::cout << "First image " << oss.str() << ".png" << std::endl; }
					if (found_first) break;
				}
				if (!found_first) { index_load++; continue; }

				if (bufferSpecPtr[buff_write]->img->data() == NULL)
				{
					boost::unique_lock<boost::mutex> l(mutex_data);
					no_more_data = true;
					//std::cout << "No more images to read." << std::endl;
					break;
				}
				if (binary)
				{
					do {
						fb.read((char*)&findex, sizeof(unsigned));
						fb.read((char*)&(bufferSpecPtr[buff_write]->timestamp), sizeof(double));
					} while (findex < index_load && !fb.eof());
				}
				else
				{
					std::fstream f((oss.str() + std::string(".time")).c_str(), std::ios_base::in);
					f >> bufferSpecPtr[buff_write]->timestamp; f.close();
				}
				index_load++;
				break;
			}
			if (no_more_data) break;
			incWritePos();
			if (condition) condition->setAndNotify(1);
		}

		if (binary) fb.close();

		JFR_GLOBAL_CATCH
	}


	void HardwareSensorCamera::init(Mode mode, std::string dump_path, cv::Size imgSize)
	{
		this->mode = mode;
		this->dump_path = dump_path;

		// configure data
		bufferImage.resize(bufferSize);
		bufferSpecPtr.resize(bufferSize);
		for(int i = 0; i < bufferSize; ++i)
		{
			bufferImage[i] = cvCreateImage(imgSize, 8, 1);
			buffer(i).reset(new RawImage());
			bufferSpecPtr[i] = SPTR_CAST<RawImage>(buffer(i));
			bufferSpecPtr[i]->setJafarImage(jafarImage_ptr_t(new image::Image(bufferImage[i])));
		}
		
		found_first = 0;
		index_load = 0;
	}


	void HardwareSensorCamera::start()
	{
		if (started) { std::cout << "Warning: This HardwareSensorCameraFirewire has already been started" << std::endl; return; }

		// start acquire task
		last_timestamp = kernel::Clock::getTime();
		if (mode == mOffline)
			preloadTask_thread = new boost::thread(boost::bind(&HardwareSensorCamera::preloadTaskOffline,this));
		else
			preloadTask_thread = new boost::thread(boost::bind(&HardwareSensorCamera::preloadTask,this));

		started = true;
	}

	void HardwareSensorCamera::stop()
	{
		if (!started) return;
		stopping = true;
		cond_offline_freed.notify_all();
	}

	void HardwareSensorCamera::join()
	{
		if (boost::this_thread::get_id() != preloadTask_thread->get_id())
			preloadTask_thread->join();
	}

	HardwareSensorCamera::HardwareSensorCamera(kernel::VariableCondition<int> *condition, cv::Size imgSize, std::string dump_path):
		HardwareSensorExteroAbstract(condition, mOffline, 3), id_(cameraIds.getId())
	{
		init(mOffline, dump_path, imgSize);
	}

	HardwareSensorCamera::HardwareSensorCamera(kernel::VariableCondition<int> *condition, Mode mode, int bufferSize, kernel::LoggerTask *loggerTask):
		HardwareSensorExteroAbstract(condition, mode, bufferSize), loggerTask(loggerTask), id_(cameraIds.getId())
	{}

	

}}}

