/**
 * \file hardwareSensorGpsGenom.cpp
 *
 * File for getting data from the genom gps module
 *
 * \date 16/03/2011
 * \author croussil
 *
 * \ingroup rtslam
 */

#include "kernel/timingTools.hpp"
#include "rtslam/hardwareSensorMocap.hpp"


namespace jafar {
namespace rtslam {
namespace hardware {


	void HardwareSensorMocap::preloadTask(void)
	{ JFR_GLOBAL_TRY
		int format, size;

		if (mode == mOnlineDump) log.openWrite("mocap", dump_path, 0, reading.data.size(), loggerTask);
		if (mode == mOffline) log.openRead("mocap", dump_path, format, size);
		
		while (!stopping)
		{
			if (mode == mOffline)
			{
				bool eof = log.read(reading.data);
				boost::unique_lock<boost::mutex> l(mutex_data);
				if (isFull(true)) cond_offline_full.notify_all();
				if (eof) { no_more_data = true; cond_offline_full.notify_all(); stopping = true; }
				while (isFull(true) && !stopping) cond_offline_freed.wait(l);
				if (stopping) break;
			} else
			{
				// TODO read data from sensor and put it in reading
				reading.arrival = kernel::Clock::getTime();
				//arrival_delay = reading.arrival - reading.data(0);
			}
			
			if (enabled)
			{
				int buff_write = getWritePos(true); // don't need to lock because we are the only writer
				buffer(buff_write).data = reading.data;
				buffer(buff_write).data(0) += timestamps_correction;
				last_timestamp = reading.data(0);
				incWritePos();
			}

			if (mode == mOnlineDump) log.write(reading.data);
		}

		if (mode == mOnlineDump || mode == mOffline) log.close();

		JFR_GLOBAL_CATCH
	}
	
	
	HardwareSensorMocap::HardwareSensorMocap(kernel::VariableCondition<int> *condition, unsigned bufferSize, Mode mode, std::string dump_path, kernel::LoggerTask *loggerTask):
		HardwareSensorProprioAbstract(condition, mode, bufferSize, ctVar), dump_path(dump_path), loggerTask(loggerTask)
	{
		if (mode == mOnlineDump && !loggerTask) JFR_ERROR(RtslamException, RtslamException::GENERIC_ERROR, "HardwareSensorMocap: you must provide a loggerTask if you want to dump data.");
		addQuantity(qPos);
		addQuantity(qOriEuler); // using euler x/y/z because the sensors work with euler and the uncertainty is provided with euler)
		initData();

		// configure
		if (mode == mOnline || mode == mOnlineDump)
		{
		}
	}

	void HardwareSensorMocap::start()
	{
		// start acquire task
		//preloadTask();
		if (started) { std::cout << "Warning: This HardwareSensorMocap has already been started" << std::endl; return; }
		started = true;
		last_timestamp = kernel::Clock::getTime();
		preloadTask_thread = new boost::thread(boost::bind(&HardwareSensorMocap::preloadTask,this));
		
		if (mode == mOffline)
		{ // wait that log has been read before first frame
			boost::unique_lock<boost::mutex> l(mutex_data);
			cond_offline_full.wait(l);
		}
	}

	void HardwareSensorMocap::stop()
	{
		if (!started) return;
		stopping = true;
		cond_offline_freed.notify_all();
	}

	void HardwareSensorMocap::join()
	{
		if (boost::this_thread::get_id() != preloadTask_thread->get_id())
			preloadTask_thread->join();
	}

}}}

