/**
 * \file mapObject.cpp
 * \date 13/03/2010
 * \author jsola
 * \ingroup rtslam
 */

#include "rtslam/mapObject.hpp"
#include "jmath/indirectArray.hpp"

namespace jafar {
	namespace rtslam {
		using namespace std;


		/*
		 * Operator << for class MapObject.
		 * It shows different information of the object.
		 */
		ostream& operator <<(ostream & s, MapObject const & obj) {
			s << obj.categoryName() << " " << obj.id() << ": ";
			if (obj.name().size() > 0)
				s << obj.name() << ", ";
			s << ".state:  " << obj.state;
			return s;
		}


		/**
		 * Selectable constructor with inFilter flag.
		 */
		MapObject::MapObject(const map_ptr_t & _mapPtr, const size_t _size, const size_t _sizeU):
			state(_mapPtr->x(), _mapPtr->P(), jmath::ublasExtra::ia_set(0,0))
		{
				category = MAPPABLE_OBJECT;
				construct(_mapPtr, _size, _sizeU);
		}

		void MapObject::construct(const map_ptr_t & _mapPtr, const size_t _size, const size_t _sizeU)
		{
			if (_size  != 0) state .extend(_mapPtr->reserveStates(_size));
			if (_sizeU != 0) stateU.extend(_sizeU);
		}

		MapObject::MapObject(const map_ptr_t & _mapPtr, const MapObject & prevObj, const size_t _size, jblas::ind_array & _icomp) :
			ObjectAbstract(),
			state( _mapPtr->x(), _mapPtr->P(),
					_mapPtr->convertStates(prevObj.state.ia(),_size,_icomp) )
		{
				category = MAPPABLE_OBJECT;
		}

	}
}
